@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.85 2004/11/15 17:21:43 sampie Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.88 2005/02/07 16:38:58 richb Exp $
  *
- *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -67,8 +67,8 @@ char *hyp_desc = N_("Set hyperbolic option for trigonometric functions");
 char *inv_desc = N_("Set inverse option for trigonometric functions");
 
 char *mstrs[] = {              /* Mode titles to be added to the titlebar. */
-    N_("Basic Mode"), N_("Advanced Mode"), N_("Financial Mode"), 
-    N_("Scientific Mode"), N_("Expression Mode")
+    N_("Basic"), N_("Advanced"), N_("Financial"), 
+    N_("Scientific"), N_("Expression")
 };
 
 char *ttype_str[] = {          /* Strings for each trig type value. */
@@ -846,8 +846,8 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     N_("Con"),
     N_("Constants [#]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },   
-    { GDK_numbersign, 0 },   
+    { GDK_SHIFT_MASK, 0,              0 },
+    { GDK_numbersign, GDK_numbersign, 0 },   
     '#',
     M_CON,            
     do_pending,
