@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.144 2004/10/15 19:20:48 sampie Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.156 2005/02/07 16:38:58 richb Exp $
  *
- *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,20 +36,35 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
+#include <gnome.h>
 #include <gconf/gconf-client.h>
 #include "ce_parser.h"
 
-#define BUT_0     X->bas_buttons[24]       /* 0 */
-#define BUT_1     X->bas_buttons[16]       /* 1 */
-#define BUT_2     X->bas_buttons[17]       /* 2 */
-#define BUT_3     X->bas_buttons[18]       /* 3 */
-#define BUT_4     X->bas_buttons[8]        /* 4 */
-#define BUT_5     X->bas_buttons[9]        /* 5 */
-#define BUT_6     X->bas_buttons[10]       /* 6 */
-#define BUT_7     X->bas_buttons[0]        /* 7 */
-#define BUT_8     X->bas_buttons[1]        /* 8 */
-#define BUT_9     X->bas_buttons[2]        /* 9 */
-#define BUT_CLR   X->bas_buttons[7]        /* Clr */
+#define BUT_0_BAS     X->bas_buttons[24]       /* 0 - in "Basic" mode */
+#define BUT_1_BAS     X->bas_buttons[16]       /* 1 - in "Basic" mode */
+#define BUT_2_BAS     X->bas_buttons[17]       /* 2 - in "Basic" mode */
+#define BUT_3_BAS     X->bas_buttons[18]       /* 3 - in "Basic" mode */
+#define BUT_4_BAS     X->bas_buttons[8]        /* 4 - in "Basic" mode */
+#define BUT_5_BAS     X->bas_buttons[9]        /* 5 - in "Basic" mode */
+#define BUT_6_BAS     X->bas_buttons[10]       /* 6 - in "Basic" mode */
+#define BUT_7_BAS     X->bas_buttons[0]        /* 7 - in "Basic" mode */
+#define BUT_8_BAS     X->bas_buttons[1]        /* 8 - in "Basic" mode */
+#define BUT_9_BAS     X->bas_buttons[2]        /* 9 - in "Basic" mode */
+
+#define BUT_0_ADV     X->adv_buttons[24]       /* 0 - in "Advanced" mode */
+#define BUT_1_ADV     X->adv_buttons[16]       /* 1 - in "Advanced" mode */
+#define BUT_2_ADV     X->adv_buttons[17]       /* 2 - in "Advanced" mode */
+#define BUT_3_ADV     X->adv_buttons[18]       /* 3 - in "Advanced" mode */
+#define BUT_4_ADV     X->adv_buttons[8]        /* 4 - in "Advanced" mode */
+#define BUT_5_ADV     X->adv_buttons[9]        /* 5 - in "Advanced" mode */
+#define BUT_6_ADV     X->adv_buttons[10]       /* 6 - in "Advanced" mode */
+#define BUT_7_ADV     X->adv_buttons[0]        /* 7 - in "Advanced" mode */
+#define BUT_8_ADV     X->adv_buttons[1]        /* 8 - in "Advanced" mode */
+#define BUT_9_ADV     X->adv_buttons[2]        /* 9 - in "Advanced" mode */
+
+#define BUT_CLR_BAS   X->bas_buttons[2]        /* Clr - in "Basic" mode */
+#define BUT_CLR_ADV   X->adv_buttons[7]        /* Clr - in "Advanced" mode */
+
 #define BUT_ACC   X->sci_buttons[7]        /* a */
 #define BUT_A     X->sci_buttons[24]       /* A */
 #define BUT_B     X->sci_buttons[25]       /* B */
@@ -88,15 +103,18 @@ struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *rframe;                 /* Register window. */
     GtkWidget *spframe;                /* Set Precision window. */
     GtkWidget *spframe_val;
+    GtkWidget *scrolledwindow;         /* Scrolled window for display_item. */
     GtkWidget *regs[MAXREGS];          /* Memory registers. */
     GtkWidget *menus[MAXMENUS];
     GtkWidget *trig[MAXTRIGMODES];     /* Trigonometric mode. */
 
     GtkWidget *bas_buttons[BROWS * BCOLS];
+    GtkWidget *adv_buttons[AROWS * ACOLS];
     GtkWidget *fin_buttons[FROWS * FCOLS];
     GtkWidget *sci_buttons[SROWS * SCOLS];
 
     GtkWidget *bas_panel;
+    GtkWidget *adv_panel;
     GtkWidget *fin_panel;
     GtkWidget *sci_panel;
 
@@ -141,7 +159,7 @@ static gboolean dismiss_spframe(GtkWidget *, GdkEvent *, gpointer);
 static gboolean kframe_key_press_cb(GtkWidget *, GdkEventKey *, gpointer);
 static gboolean spframe_key_cb(GtkWidget *, GdkEventKey *, gpointer);
 
-static void about_cb(GtkAction *action);
+static void about_cb(GtkAction *action, gpointer);
 static void add_cf_column(GtkTreeView *, gchar *, gint, gboolean);
 static void aframe_cancel_cb(GtkButton *, gpointer);
 static void aframe_ok_cb(GtkButton *, gpointer);
@@ -166,11 +184,14 @@ static void put_constant(int, char *, char *);
 static void put_function(int, char *, char *);
 static void quit_cb(GtkWidget *, gpointer);
 static void reset_mode_values(enum mode_type);
+static void set_accuracy_toggle(int val);
 static void set_button_state(GtkWidget *, int);
 static void set_gcalctool_icon(void);
+static void set_item(enum item_type itemtype, int val);
 static void set_memory_toggle(int);
 static void set_show_tsep_toggle(int);
 static void set_show_zeroes_toggle(int);
+static void show_ascii_frame();
 static void show_precision_frame();
 static void spframe_cancel_cb(GtkButton *, gpointer);
 static void spframe_ok_cb(GtkButton *, gpointer);
@@ -206,7 +227,7 @@ static GtkActionEntry entries[] = {
 
     { "Contents", GTK_STOCK_HELP, N_("_Contents"), "F1",
       N_("Show help contents"), G_CALLBACK(mb_proc) },
-    { "About", GNOME_STOCK_ABOUT, N_("_About"), NULL,
+    { "About", GTK_STOCK_ABOUT, N_("_About"), NULL,
       N_("Show the About Gcalctool dialog"), G_CALLBACK(about_cb) },
 
     { "LSPlaces1",  NULL, N_("1 place"),   NULL, 
@@ -314,12 +335,14 @@ static GtkRadioActionEntry acc_radio_entries[] = {
 static guint n_acc_radio_entries = G_N_ELEMENTS(acc_radio_entries);
 
 static GtkRadioActionEntry base_radio_entries[] = {
-  { "Basic",	  NULL, N_("_Basic mode"),	"<control>B",
-    N_("Basic mode"),	   M_BASIC },
-  { "Financial",  NULL, N_("_Financial mode"),	"<control>F",
-    N_("Financial mode"),  M_FIN },
-  { "Scientific", NULL, N_("_Scientific mode"), "<control>S",
-    N_("Scientific mode"), M_SCI },
+  { "Basic",      NULL, N_("_Basic"),      "<control>B", 
+    N_("Basic"),      M_BASIC },
+  { "Advanced",   NULL, N_("_Advanced"),   "<control>A",
+    N_("Advanced"),   M_ADV },
+  { "Financial",  NULL, N_("_Financial"),  "<control>F",
+    N_("Financial"),  M_FIN },
+  { "Scientific", NULL, N_("_Scientific"), "<control>S",
+    N_("Scientific"), M_SCI },
 };
 
 static guint n_base_radio_entries = G_N_ELEMENTS(base_radio_entries);
@@ -338,6 +361,7 @@ static const gchar *ui_info =
 "    </menu>"
 "    <menu action='ViewMenu'>"
 "      <menuitem action='Basic'/>"
+"      <menuitem action='Advanced'/>"
 "      <menuitem action='Financial'/>"
 "      <menuitem action='Scientific'/>"
 "      <separator/>"
@@ -465,11 +489,9 @@ update_statusbar(gchar *text, const gchar *imagename)
 
 /*ARGSUSED*/
 static void
-about_cb(GtkAction *action)
+about_cb(GtkAction *action, gpointer date)
 {
-    static GtkWidget *about = NULL;
-
-    if (about == NULL) {
+    
         const gchar *authors[] = {
             "Rich Burridge <rich.burridge@sun.com>",
             "Sami Pietila <sampie@ariana-dsl.utu.fi>",
@@ -481,21 +503,17 @@ about_cb(GtkAction *action)
         };
         const gchar *translator_credits = _("translator_credits");
 
-        about = gnome_about_new(_("Gcalctool"),
-                       VERSION,
-                       "(C) 2004 the Free Software Foundation",
-                       _("Calculator with financial and scientific modes."),
-                       authors,
-                       documenters,
-                       strcmp(translator_credits, "translator_credits") != 0 ? 
-                              translator_credits : NULL,
-                       X->icon);
-
-        g_signal_connect(G_OBJECT(about), "destroy",
-                         G_CALLBACK(gtk_widget_destroyed), &about);
-        gtk_window_set_icon(GTK_WINDOW(about), X->icon);    
-    }
-    gtk_window_present(GTK_WINDOW(about));
+        gtk_show_about_dialog(GTK_WINDOW (X->kframe),
+        					"name",_("Gcalctool"),
+        					"version", VERSION,
+        					"copyright", "(C) 2004 the Free Software Foundation",
+        					"comments", _("Calculator with financial and scientific modes."),
+        					"authors", authors,
+        					"documenters", documenters,
+        					"translator-credits", strcmp(translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+        					"logo", X->icon,
+                                                NULL);
+        					
 }
 
 
@@ -619,7 +637,7 @@ button_proc(GtkButton *widget, gpointer user_data)
             }
             if (v->new_input && v->dtype == FIX) {
                 STRCPY(v->fnum, v->display);
-                set_display(v->fnum, FALSE);
+                set_display(v->fnum, TRUE);
             }
             break;
 
@@ -1086,7 +1104,6 @@ create_kframe()
     GError *error;
     GtkWidget *event_box, *view_widget;
     GtkTextBuffer *buffer;
-    GtkWidget *scrolledwindow;
 
     v->tool_label = NULL;
     if (v->titleline == NULL) {
@@ -1101,7 +1118,7 @@ create_kframe()
 
     X->kframe = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     g_object_set_data(G_OBJECT(X->kframe), "kframe", X->kframe);
-    gtk_window_set_resizable(GTK_WINDOW(X->kframe), FALSE);
+    gtk_window_set_resizable(GTK_WINDOW(X->kframe), TRUE);
 
     g_signal_connect(G_OBJECT(X->kframe), "delete_event",
                        G_CALLBACK(quit_cb), NULL);
@@ -1141,10 +1158,10 @@ create_kframe()
     gtk_box_pack_start(GTK_BOX(X->kvbox), X->menubar, FALSE, FALSE, 0);
 
 
-    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
-    gtk_widget_show(scrolledwindow);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow), 
-				   GTK_POLICY_ALWAYS, 
+    X->scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
+    gtk_widget_show(X->scrolledwindow);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (X->scrolledwindow), 
+				   GTK_POLICY_AUTOMATIC, 
 				   GTK_POLICY_NEVER);
 
 
@@ -1167,14 +1184,14 @@ create_kframe()
     gtk_text_view_set_pixels_below_lines(GTK_TEXT_VIEW(X->display_item), 8);
     gtk_text_view_set_right_margin(GTK_TEXT_VIEW(X->display_item), 6);
 
-    gtk_container_add(GTK_CONTAINER(scrolledwindow), X->display_item);
+    gtk_container_add(GTK_CONTAINER(X->scrolledwindow), X->display_item);
     atk_object_set_role(gtk_widget_get_accessible(X->display_item), 
                                                   ATK_ROLE_EDITBAR);
     set_display("0.00", FALSE);
 
     gtk_widget_ref(X->display_item);
     gtk_container_set_border_width(GTK_CONTAINER(X->display_item), 2);
-    gtk_container_add(GTK_CONTAINER(event_box), scrolledwindow);
+    gtk_container_add(GTK_CONTAINER(event_box), X->scrolledwindow);
     gtk_widget_show(X->display_item);
     gtk_box_pack_start(GTK_BOX(X->kvbox), event_box, FALSE, TRUE, 0);
     gtk_widget_show(event_box);
@@ -1198,23 +1215,28 @@ create_kframe()
     X->bas_panel = make_but_panel(X->kvbox, X->bas_buttons,
                                   &b_buttons[0], BROWS, BCOLS, "bas");
     gtk_widget_show(X->bas_panel);
+
+    X->adv_panel = make_but_panel(X->kvbox, X->adv_buttons,
+                                  &a_buttons[0], AROWS, ACOLS, "adv");
+    gtk_widget_show(X->adv_panel);
     gtk_window_add_accel_group(GTK_WINDOW(X->kframe), X->kbd_accel);
     grey_buttons(v->base);
     gtk_window_set_icon(GTK_WINDOW(X->kframe), X->icon);
-    gtk_window_set_focus(GTK_WINDOW(X->kframe), GTK_WIDGET(BUT_CLR));
+    if (v->modetype == BASIC) {
+        gtk_window_set_focus(GTK_WINDOW(X->kframe), GTK_WIDGET(BUT_CLR_BAS));
+    } else {
+        gtk_window_set_focus(GTK_WINDOW(X->kframe), GTK_WIDGET(BUT_CLR_ADV));
+    }
 
-    X->khbox = gtk_hbox_new(FALSE, 0);
-    gtk_widget_show(X->khbox);
-    gtk_container_add(GTK_CONTAINER(X->kvbox), X->khbox);
     X->statusbar = gtk_statusbar_new();
     gtk_widget_show(X->statusbar);
-    gtk_box_pack_start(GTK_BOX(X->khbox), X->statusbar, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(X->kvbox), X->statusbar, FALSE, FALSE, 0);
     gtk_statusbar_set_has_resize_grip(GTK_STATUSBAR(X->statusbar), FALSE);
 
     X->status_image = gtk_image_new_from_stock("", 
 					       GTK_ICON_SIZE_BUTTON);
     gtk_widget_show(X->status_image);
-    gtk_box_pack_start(GTK_BOX(X->khbox), X->status_image, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(X->statusbar), X->status_image, FALSE, TRUE, 0);
     g_signal_connect(G_OBJECT(X->kframe), "key_press_event",
                      G_CALLBACK(kframe_key_press_cb), NULL);
 
@@ -1229,6 +1251,11 @@ create_kframe()
                                               "/MenuBar/ViewMenu/Scientific");
             break;
 
+        case ADVANCED:
+            view_widget = gtk_ui_manager_get_widget(X->ui,
+                                              "/MenuBar/ViewMenu/Advanced");
+            break;
+
         default:
             view_widget = gtk_ui_manager_get_widget(X->ui, 
                                               "/MenuBar/ViewMenu/Basic");
@@ -1671,16 +1698,30 @@ get_resource(enum res_type rtype)
 void
 grey_buttons(enum base_type base)
 {
-    set_button_state(BUT_0, (0 < basevals[(int) base]));
-    set_button_state(BUT_1, (1 < basevals[(int) base]));
-    set_button_state(BUT_2, (2 < basevals[(int) base]));
-    set_button_state(BUT_3, (3 < basevals[(int) base]));
-    set_button_state(BUT_4, (4 < basevals[(int) base]));
-    set_button_state(BUT_5, (5 < basevals[(int) base]));
-    set_button_state(BUT_6, (6 < basevals[(int) base]));
-    set_button_state(BUT_7, (7 < basevals[(int) base]));
-    set_button_state(BUT_8, (8 < basevals[(int) base]));
-    set_button_state(BUT_9, (9 < basevals[(int) base]));
+    if (v->modetype == BASIC) {
+        set_button_state(BUT_0_BAS, (0 < basevals[(int) base]));
+        set_button_state(BUT_1_BAS, (1 < basevals[(int) base]));
+        set_button_state(BUT_2_BAS, (2 < basevals[(int) base]));
+        set_button_state(BUT_3_BAS, (3 < basevals[(int) base]));
+        set_button_state(BUT_4_BAS, (4 < basevals[(int) base]));
+        set_button_state(BUT_5_BAS, (5 < basevals[(int) base]));
+        set_button_state(BUT_6_BAS, (6 < basevals[(int) base]));
+        set_button_state(BUT_7_BAS, (7 < basevals[(int) base]));
+        set_button_state(BUT_8_BAS, (8 < basevals[(int) base]));
+        set_button_state(BUT_9_BAS, (9 < basevals[(int) base]));
+    } else {
+        set_button_state(BUT_0_ADV, (0 < basevals[(int) base]));
+        set_button_state(BUT_1_ADV, (1 < basevals[(int) base]));
+        set_button_state(BUT_2_ADV, (2 < basevals[(int) base]));
+        set_button_state(BUT_3_ADV, (3 < basevals[(int) base]));
+        set_button_state(BUT_4_ADV, (4 < basevals[(int) base]));
+        set_button_state(BUT_5_ADV, (5 < basevals[(int) base]));
+        set_button_state(BUT_6_ADV, (6 < basevals[(int) base]));
+        set_button_state(BUT_7_ADV, (7 < basevals[(int) base]));
+        set_button_state(BUT_8_ADV, (8 < basevals[(int) base]));
+        set_button_state(BUT_9_ADV, (9 < basevals[(int) base]));
+    }
+
     set_button_state(BUT_A, (10 < basevals[(int) base]));
     set_button_state(BUT_B, (11 < basevals[(int) base]));
     set_button_state(BUT_C, (12 < basevals[(int) base]));
@@ -1749,12 +1790,12 @@ check_vals(int n, int keyval, int state,
 {
     int i, j;
 
-    state = state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK);
+    state = state & (GDK_CONTROL_MASK | GDK_MOD1_MASK);
     for (i = 0; i < n; i++) {
         j = 0;
         while (buttons[i].value[j] != 0) {
             if (buttons[i].value[j] == keyval) {
-                if (buttons[i].mods[j] == state) {
+                if ((buttons[i].mods[j] & ~GDK_SHIFT_MASK) == state) {
                     button_proc(GTK_BUTTON(gtk_buttons[i]), NULL);
                     return(TRUE);
                 }
@@ -1783,9 +1824,14 @@ kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
                                 b_buttons, X->bas_buttons);
             break;
 
+        case ADVANCED:
+            retval = check_vals(A_NOBUTTONS, event->keyval, event->state,
+                                a_buttons, X->adv_buttons);
+            break;
+
         case FINANCIAL:
-            retval = check_vals(B_NOBUTTONS, event->keyval, event->state,
-            			b_buttons, X->bas_buttons);
+            retval = check_vals(A_NOBUTTONS, event->keyval, event->state,
+            			a_buttons, X->adv_buttons);
             if (retval != TRUE) {
                 retval = check_vals(F_NOBUTTONS, event->keyval, event->state,
                                     f_buttons, X->fin_buttons);
@@ -1793,8 +1839,8 @@ kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
             break;
 
         case SCIENTIFIC:
-            retval = check_vals(B_NOBUTTONS, event->keyval, event->state,
-                                b_buttons, X->bas_buttons);
+            retval = check_vals(A_NOBUTTONS, event->keyval, event->state,
+                                a_buttons, X->adv_buttons);
             if (retval != TRUE) {
                 retval = check_vals(S_NOBUTTONS, event->keyval, event->state,
                                     s_buttons, X->sci_buttons);
@@ -2021,11 +2067,11 @@ menu_proc_cb(GtkMenuItem *mi, gpointer user_data)
 
 /*ARGSUSED*/
 static void 
-menu_button_button_press_cb(GtkWidget *widget, 
-                            GdkEventButton *event, gpointer data)
+menu_button_button_press_cb(GtkButton *widget, gpointer data)
 {
     struct button *n;
     GtkWidget *menu;
+    GdkEventButton *event = (GdkEventButton *) gtk_get_current_event();
 
     if (event->button != 1) {
         return;
@@ -2089,7 +2135,7 @@ make_menu_button(gchar *label_text, int n)
     gtk_container_add(GTK_CONTAINER(button), hbox);
  
     gtk_widget_set_events(button, GDK_BUTTON_PRESS_MASK);
-    g_signal_connect(G_OBJECT(button), "button-press-event",
+    g_signal_connect(G_OBJECT(button), "clicked",
                      G_CALLBACK(menu_button_button_press_cb), (gpointer) n);
     g_signal_connect(G_OBJECT(button), "key-press-event",
                      G_CALLBACK(menu_button_key_press_cb), (gpointer) n);
@@ -2181,6 +2227,8 @@ mb_base_radio_proc(GtkAction *action, GtkRadioAction *current)
 
     if (EQUAL(name, "Basic")) {
         reset_mode_values(BASIC);
+    } else if (EQUAL(name, "Advanced")) {
+        reset_mode_values(ADVANCED);
     } else if (EQUAL(name, "Financial")) {
         reset_mode_values(FINANCIAL);
     } else if (EQUAL(name, "Scientific")) {
@@ -2391,6 +2439,22 @@ set_accuracy_toggle(int val)
 }
 
 
+void
+scroll_right()
+{
+    if (GTK_WIDGET_VISIBLE(
+                   GTK_SCROLLED_WINDOW(X->scrolledwindow)->hscrollbar)) {
+        GtkAdjustment *set;
+
+        set = gtk_scrolled_window_get_hadjustment(
+                                GTK_SCROLLED_WINDOW(X->scrolledwindow));
+        gtk_adjustment_set_value(set, set->upper);
+        gtk_scrolled_window_set_hadjustment(
+                                GTK_SCROLLED_WINDOW(X->scrolledwindow), set);
+    }
+}
+
+
 void
 set_display(char *str, int minimize_changes)
 {
@@ -2449,14 +2513,14 @@ set_display(char *str, int minimize_changes)
                                                      NULL);
         }
     }
-    g_free (text);
+    scroll_right();
+    g_free(text);
 }
 
 
 void
 write_display(char *str)
 {
-    char localized[MAX_LOCALIZED];
     gchar *text;
     GtkTextBuffer *buffer;
     GtkTextIter start, end;
@@ -2475,8 +2539,8 @@ write_display(char *str)
 					     -1,
 					     "x-large",
 					     NULL);
+    scroll_right();
     g_free(text);
-
 }
 
 #define SET_MENUBAR_ITEM_STATE(i, state) \
@@ -2506,7 +2570,14 @@ set_error_state(int error)
     for (i = 0; i < (BROWS * BCOLS); i++) {
         set_button_state(X->bas_buttons[i], !v->error);
     }
-    set_button_state(BUT_CLR, TRUE);    /* Clr button always sensitive. */
+    /* Clr ("Basic") button always sensitive. */
+    set_button_state(BUT_CLR_BAS, TRUE);
+
+    for (i = 0; i < (AROWS * ACOLS); i++) {
+        set_button_state(X->adv_buttons[i], !v->error);
+    }
+    /* Clr ("Advanced") button always sensitive. */
+    set_button_state(BUT_CLR_ADV, TRUE);
 
     for (i = 0; i < (FROWS * FCOLS); i++) {
         set_button_state(X->fin_buttons[i], !v->error);
@@ -2526,6 +2597,7 @@ set_error_state(int error)
     SET_MENUBAR_ITEM_STATE("/MenuBar/EditMenu/Paste",      !v->error); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/EditMenu/Insert",     !v->error); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Basic",      !v->error); 
+    SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Advanced",   !v->error); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Financial",  !v->error); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Scientific", !v->error); 
     SET_MENUBAR_ITEM_STATE("/MenuBar/ViewMenu/Trailing",
@@ -2569,18 +2641,32 @@ set_mode(enum mode_type mode)
   
     switch (mode) {
         case BASIC:
+            gtk_widget_show(X->bas_panel);
+            gtk_widget_hide(X->adv_panel);
+            gtk_widget_hide(X->fin_panel);
+            gtk_widget_hide(X->mode_panel);
+            gtk_widget_hide(X->sci_panel);
+            break;
+
+        case ADVANCED:
+            gtk_widget_hide(X->bas_panel);
+            gtk_widget_show(X->adv_panel);
             gtk_widget_hide(X->fin_panel);
             gtk_widget_hide(X->mode_panel);
             gtk_widget_hide(X->sci_panel);
             break;
 
         case FINANCIAL:
+            gtk_widget_hide(X->bas_panel);
+            gtk_widget_show(X->adv_panel);
             gtk_widget_show(X->fin_panel);
             gtk_widget_hide(X->mode_panel);
             gtk_widget_hide(X->sci_panel);
             break;
 
         case SCIENTIFIC:
+            gtk_widget_hide(X->bas_panel);
+            gtk_widget_show(X->adv_panel);
             gtk_widget_hide(X->fin_panel);
             gtk_widget_show_all(X->mode_panel);
             gtk_widget_show(X->sci_panel);
